@@ -123,16 +123,18 @@ setMethod("show",
 #' \code{\link{SimParam}}.
 #' 
 #' @param x a 'MapPop' object
-#' @param i index of chromosomes
-#' @param ... aditional 'MapPop' objects
+#' @param i index of individuals
+#' @param ... additional 'MapPop' objects
 #' 
-#' @slot genMap "matrix" of chromsome genetic maps
+#' @slot genMap "matrix" of chromosome genetic maps
 #' @slot centromere vector of centromere positions
+#' @slot inbred indicates whether the individuals are fully inbred
 #' 
 #' @export
 setClass("MapPop",
-         slots=c(genMap="matrix",
-                 centromere="numeric"),
+         slots=c(genMap="list",
+                 centromere="numeric",
+                 inbred="logical"),
          contains="RawPop")
 
 setValidity("MapPop",function(object){
@@ -164,7 +166,6 @@ setMethod("[",
               x@geno[[chr]] = x@geno[[chr]][,,i,drop=FALSE]
             }
             x@nInd = dim(x@geno[[1]])[3]
-            class(x) = "MapPop"
             return(x)
           }
 )
@@ -184,6 +185,99 @@ setMethod("c",
                           all.equal(x@genMap, y@genMap))
                 x@nInd = x@nInd+y@nInd
                 x@geno = mergeGeno(x@geno,y@geno)
+                x@inbred = x@inbred & y@inbred
+              }
+            }
+            return(x)
+          }
+)
+
+# NamedMapPop ------------------------------------------------------------------
+
+#' @title Raw population with genetic map and id
+#' 
+#' @description 
+#' Extends \code{\link{MapPop-class}} to add id, mother and father. 
+#' 
+#' @param x a 'NamedMapPop' object
+#' @param i index of individuals
+#' @param ... additional 'NamedMapPop' objects
+#' 
+#' @slot id an individual's identifier
+#' @slot mother the identifier of the individual's mother
+#' @slot father the identifier of the individual's father
+#' 
+#' @export
+setClass("NamedMapPop",
+         slots=c(id="character",
+                 mother="character",
+                 father="character"),
+         contains="MapPop")
+
+setValidity("NamedMapPop",function(object){
+  errors = character()
+  if(any(grepl(" ",object@id,fixed=TRUE))){
+    errors = c(errors,"id can not contain spaces")
+  }
+  if(any(grepl(" ",object@mother,fixed=TRUE))){
+    errors = c(errors,"mother can not contain spaces")
+  }
+  if(any(grepl(" ",object@father,fixed=TRUE))){
+    errors = c(errors,"father can not contain spaces")
+  }
+  if(object@nInd!=length(object@id)){
+    errors = c(errors,"nInd!=length(id)")
+  }
+  if(object@nInd!=length(object@mother)){
+    errors = c(errors,"nInd!=length(mother)")
+  }
+  if(object@nInd!=length(object@father)){
+    errors = c(errors,"nInd!=length(father)")
+  }
+  if(length(errors)==0){
+    return(TRUE)
+  }else{
+    return(errors)
+  }
+})
+
+#' @describeIn NamedMapPop Extract NamedMapPop by index
+setMethod("[",
+          signature(x = "NamedMapPop"),
+          function(x, i){
+            if(any(abs(i)>x@nInd)){
+              stop("Trying to select invalid individuals")
+            }
+            for(chr in 1:x@nChr){
+              x@geno[[chr]] = x@geno[[chr]][,,i,drop=FALSE]
+            }
+            x@nInd = dim(x@geno[[1]])[3]
+            x@id = x@id[i]
+            x@mother = x@mother[i]
+            x@father = x@father[i]
+            return(x)
+          }
+)
+
+#' @describeIn NamedMapPop Combine multiple NamedMapPops 
+setMethod("c",
+          signature(x = "NamedMapPop"),
+          function (x, ...){
+            for(y in list(...)){
+              if(class(y)=="NULL"){
+                # Do nothing
+              }else{
+                stopifnot(class(y)=="NamedMapPop",
+                          x@nChr==y@nChr,
+                          x@ploidy==y@ploidy,
+                          x@nLoci==y@nLoci,
+                          all.equal(x@genMap, y@genMap))
+                x@nInd = x@nInd+y@nInd
+                x@id = c(x@id, y@id)
+                x@mother = c(x@mother, y@mother)
+                x@father = c(x@father, y@father)
+                x@geno = mergeGeno(x@geno,y@geno)
+                x@inbred = x@inbred & y@inbred
               }
             }
             return(x)
@@ -193,10 +287,12 @@ setMethod("c",
 #' @title Combine MapPop chromosomes
 #' 
 #' @description
-#' Merges the chromosomes of multiple \code{\link{MapPop-class}} objects. 
+#' Merges the chromosomes of multiple \code{\link{MapPop-class}} or 
+#' \code{\link{NamedMapPop-class}} objects. 
 #' Each MapPop must have the same number of chromosomes
 #'
-#' @param ... \code{\link{MapPop-class}} objects to be combined
+#' @param ... \code{\link{MapPop-class}} or \code{\link{NamedMapPop-class}} 
+#' objects to be combined
 #' 
 #' @return Returns an object of \code{\link{MapPop-class}}
 #' 
@@ -212,7 +308,7 @@ cChr = function(...){
     if(class(y)=="NULL"){
       #Do nothing
     }else{
-      stopifnot(class(y)=="MapPop")
+      stopifnot(class(y)=="MapPop" | class(y)=="NamedMapPop")
       if(!exists("x",inherits=FALSE)){
         x = y
       }else{
@@ -220,9 +316,10 @@ cChr = function(...){
                   x@ploidy==y@ploidy)
         x@nChr = x@nChr+y@nChr
         x@geno = rbind(x@geno,y@geno)
-        x@genMap = rbind(x@genMap,y@genMap)
+        x@genMap = c(x@genMap,y@genMap)
         x@centromere = c(x@centromere,y@centromere)
         x@nLoci = c(x@nLoci,y@nLoci)
+        x@inbred = x@inbred & y@inbred
       }
     }
   }
@@ -243,13 +340,14 @@ cChr = function(...){
 #' @param ... additional 'Pop' objects
 #' 
 #' @slot id an individual's identifier
+#' @slot iid an individual's internal identifier
 #' @slot mother the identifier of the individual's mother
 #' @slot father the identifier of the individual's father
 #' @slot sex sex of individuals: "M" for males, "F" for females,
 #' and "H" for hermaphrodites
 #' @slot nTraits number of traits
 #' @slot gv matrix of genetic values. When using GxE traits,
-#' gv reflects gv when w=0. Dimensions are nInd by nTraits.
+#' gv reflects gv when p=0.5. Dimensions are nInd by nTraits.
 #' @slot pheno matrix of phenotypic values. Dimensions are
 #' nInd by nTraits.
 #' @slot ebv matrix of estimated breeding values. Dimensions 
@@ -267,6 +365,7 @@ cChr = function(...){
 #' @export
 setClass("Pop",
          slots=c(id="character",
+                 iid="integer",
                  mother="character",
                  father="character",
                  sex="character",
@@ -297,6 +396,9 @@ setValidity("Pop",function(object){
   if(object@nInd!=length(object@id)){
     errors = c(errors,"nInd!=length(id)")
   }
+  if(object@nInd!=length(object@iid)){
+    errors = c(errors,"nInd!=length(iid)")
+  }
   if(object@nInd!=length(object@mother)){
     errors = c(errors,"nInd!=length(mother)")
   }
@@ -364,6 +466,7 @@ setMethod("[",
               }
             }
             x@id = x@id[i]
+            x@iid = x@iid[i]
             x@mother = x@mother[i]
             x@father = x@father[i]
             x@fixEff = x@fixEff[i]
@@ -423,15 +526,11 @@ setMethod("show",
 #'
 #' @param rawPop an object of \code{\link{MapPop-class}} or 
 #' \code{\link{RawPop-class}}
-#' @param mother optional id for mothers. Must match 
-#' id in pedigree if using track pedigree.
-#' @param father optional id for fathers. Must match 
-#' id in pedigree if using track pedigree.
-#' @param origM optional alternative id for mothers
-#' @param origF optional alternative id for fathers
-#' @param isDH optional value indicating if the individuals 
-#' are doubled haploids and/or inbred founders
+#' @param id optional id for new individuals.
+#' @param mother optional id for mothers.
+#' @param father optional id for fathers.
 #' @param simParam an object of \code{\link{SimParam}}
+#' @param ... additional arguments used internally
 #'
 #' @return Returns an object of \code{\link{Pop-class}}
 #' 
@@ -447,50 +546,85 @@ setMethod("show",
 #' pop = newPop(founderPop, simParam=SP)
 #' 
 #' @export
-newPop = function(rawPop,mother=NULL,father=NULL,origM=NULL,
-                  origF=NULL,isDH=FALSE,simParam=NULL){
+newPop = function(rawPop,id=NULL,mother=NULL,father=NULL,simParam=NULL,...){
   if(is.null(simParam)){
     simParam = get("SP",envir=.GlobalEnv)
   }
+  args = list(...)
   stopifnot(sapply(simParam$genMap,length)==rawPop@nLoci)
   lastId = simParam$lastId
-  id = (1:rawPop@nInd) + lastId
-  lastId = max(id)
+  iid = (1:rawPop@nInd) + lastId
+  lastId = max(iid)
+  if(is.null(id)){
+    if(class(rawPop)=="NamedMapPop"){
+      id = rawPop@id
+    }else{
+      id = as.character(iid)
+    }
+  }else{
+    id = as.character(id)
+    stopifnot(length(id)==rawPop@nInd)
+  }
+  if(any(names(args)=="iMother")){
+    iMother = args$iMother
+  }else{
+    iMother = rep(0L, rawPop@nInd)
+  }
+  if(any(names(args)=="iFather")){
+    iFather = args$iFather
+  }else{
+    iFather = rep(0L, rawPop@nInd)
+  }
+  if(any(names(args)=="isDH")){
+    isDH = args$isDH
+  }else{
+    if(class(rawPop)=="MapPop" | class(rawPop)=="NamedMapPop"){
+      isDH = rawPop@inbred
+    }else{
+      isDH = FALSE
+    }
+  }
   if(is.null(mother)){
-    mother = rep("0",rawPop@nInd)
+    if(class(rawPop)=="NamedMapPop"){
+      mother = rawPop@mother
+    }else{
+      mother = rep("0", rawPop@nInd)
+    }
   }else{
     mother = as.character(mother)
   }
   if(is.null(father)){
-    father = rep("0",rawPop@nInd)
+    if(class(rawPop)=="NamedMapPop"){
+      father = rawPop@father
+    }else{
+      father = rep("0", rawPop@nInd)
+    }
   }else{
     father = as.character(father)
   }
   stopifnot(length(id)==length(mother),
             length(id)==length(father))
   if(simParam$sexes=="no"){
-    sex = rep("H",rawPop@nInd)
+    sex = rep("H", rawPop@nInd)
   }else if(simParam$sexes=="yes_rand"){
-    sex = sample(c("M","F"),rawPop@nInd,replace=TRUE)
+    sex = sample(c("M","F"), rawPop@nInd, replace=TRUE)
   }else if(simParam$sexes=="yes_sys"){
-    sex = rep_len(c("M","F"),rawPop@nInd)
+    sex = rep_len(c("M","F"), rawPop@nInd)
   }else{
-    stop(paste("no rules for sex type",simParam$sexes))
+    stop(paste("no rules for sex type", simParam$sexes))
   }
   gxe = vector("list",simParam$nTraits)
   gv = matrix(NA_real_,nrow=rawPop@nInd,
               ncol=simParam$nTraits)
   if(simParam$nTraits>=1){
     for(i in 1:simParam$nTraits){
-      tmp = getGv(simParam$traits[[i]],rawPop,simParam$nThreads)
+      tmp = getGv(simParam$traits[[i]], rawPop, simParam$nThreads)
       gv[,i] = tmp[[1]]
       if(length(tmp)>1){
         gxe[[i]] = tmp[[2]]
       }
     }
   }
-  if(is.null(origM)) origM = mother
-  if(is.null(origF)) origF = father
   output = new("Pop",
                nInd=rawPop@nInd,
                nChr=rawPop@nChr,
@@ -498,9 +632,10 @@ newPop = function(rawPop,mother=NULL,father=NULL,origM=NULL,
                nLoci=rawPop@nLoci,
                sex=sex,
                geno=rawPop@geno,
-               id=as.character(id),
-               mother=origM,
-               father=origF,
+               id=id,
+               iid=iid,
+               mother=mother,
+               father=father,
                fixEff=rep(1L,rawPop@nInd),
                reps=rep(1,rawPop@nInd),
                nTraits=simParam$nTraits,
@@ -515,13 +650,21 @@ newPop = function(rawPop,mother=NULL,father=NULL,origM=NULL,
                misc=vector("list",rawPop@nInd))
   if(simParam$nTraits>=1){
     output = setPheno(output, varE=NULL, reps=1, 
-                      fixEff=1L, p=NULL, 
-                      onlyPheno=FALSE, 
+                      fixEff=1L, p=NULL, onlyPheno=FALSE, 
                       simParam=simParam)
   }
-  output = simParam$finalizePop(output)
+  output = simParam$finalizePop(output,...)
   if(simParam$isTrackPed){
-    simParam$addToPed(lastId,mother,father,isDH)
+    if(simParam$isTrackRec){
+      if(any(names(args)=="hist")){
+        hist = args$hist
+      }else{
+        hist = NULL
+      }
+      simParam$addToRec(lastId,id,iMother,iFather,isDH,hist,output@ploidy)
+    }else{
+      simParam$addToPed(lastId,id,iMother,iFather,isDH)
+    }
   }else{
     simParam$updateLastId(lastId)
   }
@@ -582,3 +725,111 @@ resetPop = function(pop,simParam=NULL){
   }
   return(pop)
 }
+
+# MegaPop ------------------------------------------------------------------
+
+#' @title Mega-Population
+#' 
+#' @description 
+#' The mega-population represents a population of populations. 
+#' It is designed to behave like a list of populations. 
+#' 
+#' @param x a 'MegaPop' object
+#' @param i index of populations or mega-populations
+#' @param ... additional 'MegaPop' or 'Pop' objects
+#' 
+#' @slot pops list of \code{\link{Pop-class}} and/or 
+#' \code{MegaPop-class}
+#' 
+#' 
+#' @export
+setClass("MegaPop",
+         slots=c(pops="list"))
+
+setValidity("MegaPop",function(object){
+  errors = character()
+    # Check that all populations are valid
+    for(i in 1:length(object@pops)){
+      if(!validObject(object@pops[[i]]) & 
+         (class(object@pops[[i]])=="Pop" | 
+                class(object@pops[[i]])=="MegaPop")){
+        errors = c(errors,paste("object",i,"is not a valid pop"))
+      }
+    }
+  if(length(errors)==0){
+    return(TRUE)
+  }else{
+    return(errors)
+  }
+})
+
+#' @describeIn MegaPop Extract MegaPop by index
+setMethod("[",
+          signature(x = "MegaPop"),
+          function(x, i){
+            x@pops = x@pops[i]
+            return(x)
+          }
+)
+
+#' @describeIn MegaPop Extract Pop by index
+setMethod("[[",
+          signature(x = "MegaPop"),
+          function (x, i){
+            return(x@pops[[i]])
+          }
+)
+
+#' @describeIn MegaPop Combine multiple MegaPops
+setMethod("c",
+          signature(x = "MegaPop"),
+          function (x, ...){
+            for(y in list(...)){
+              if(class(y)=="NULL"){
+                # Do nothing
+              }else{
+                if(class(y)=="Pop"){
+                  x@pops = c(x@pops, y)
+                }else{
+                  stopifnot(class(y)=="MegaPop")
+                  x@pops = c(x@pops, y@pops)
+                }
+              }
+            }
+            return(x)
+          }
+)
+
+#' @title Create new Mega Population
+#' 
+#' @description
+#' Creates a new \code{\link{MegaPop-class}} from one or more
+#' \code{\link{Pop-class}} and/or \code{\link{MegaPop-class}} 
+#' objects.  
+#'
+#' @param ... one or more \code{\link{Pop-class}} and/or 
+#' \code{\link{MegaPop-class}} objects.
+#'
+#' @return Returns an object of \code{\link{MegaPop-class}}
+#' 
+#' @examples 
+#' #Create founder haplotypes
+#' founderPop = quickHaplo(nInd=2, nChr=1, segSites=10)
+#' 
+#' #Set simulation parameters
+#' SP = SimParam$new(founderPop)
+#' SP$addTraitA(10)
+#' 
+#' #Create population
+#' pop = newPop(founderPop, simParam=SP)
+#' megaPop = newMegaPop(pop=pop)
+#' 
+#' @export
+newMegaPop = function(...){
+  input = list(...)
+  class = sapply(input, "class")
+  stopifnot(all(class=="Pop" | class=="MegaPop"))
+  output = new("MegaPop", pops=input)
+  return(output)
+}
+
